library slang_retail_assistant;

import 'dart:collection';
import 'dart:core';
import 'package:flutter/services.dart';
import 'dart:io' show Platform;

part 'src/assistant_ui_position.dart';

part 'src/assistant_config.dart';

part 'src/search_user_journey.dart';

part 'src/assistant_ui.dart';

part 'src/search_info.dart';

part 'src/navigation_user_journey.dart';

part 'src/navigation_info.dart';

part 'src/order_info.dart';

part 'src/order_management_user_journey.dart';

class SlangRetailAssistant {
  SlangRetailAssistant._();

  static const MethodChannel _channel =
      const MethodChannel('plugins.flutter.io/slang_retail_assistant');

  static late RetailAssistantAction _action;
  static late RetailAssistantLifeCycleObserver _lifeCycleObserver;
  static late SearchUserJourney _searchUserJourney;
  static late NavigationUserJourney _navigationUserJourney;
  static late OrderManagementUserJourney _orderManagementUserJourney;

  static Future<void> _methodCallHandler(MethodCall call) async {
    switch (call.method) {
      case 'onSearch':
        _searchUserJourney = new SearchUserJourney._();
        SearchAppState searchAppState = _action.onSearch(
            new SearchInfo._(call.arguments), _searchUserJourney);
        if (searchAppState != SearchAppState.WAITING) {
          _searchUserJourney.notifyAppState(searchAppState);
        }
        break;
      case 'onNavigation':
        _navigationUserJourney = new NavigationUserJourney._();
        NavigationAppState navigationAppState = _action.onNavigation(
            NavigationInfo._(call.arguments), _navigationUserJourney);
        if (navigationAppState != NavigationAppState.WAITING) {
          _navigationUserJourney.notifyAppState(navigationAppState);
        }
        break;
      case 'onOrderManagement':
        _orderManagementUserJourney = new OrderManagementUserJourney._();
        OrderManagementAppState orderManagementAppState =
            _action.onOrderManagement(
                OrderInfo._(call.arguments), _orderManagementUserJourney);
        if (orderManagementAppState != OrderManagementAppState.WAITING) {
          _orderManagementUserJourney.notifyAppState(orderManagementAppState);
        }
        break;
      case 'onAssistantError':
        final Map<dynamic, dynamic> _data = call.arguments;
        Map<String, String> arguments = new HashMap();
        arguments["description"] = _data["description"];
        _action.onAssistantError(arguments);
        break;
      case 'onAssistantInitSuccess':
        _lifeCycleObserver.onAssistantInitSuccess();
        break;
      case 'onAssistantInitFailure':
        final Map<dynamic, dynamic> _data = call.arguments;
        String description = _data["type"];
        _lifeCycleObserver.onAssistantInitFailure(description);
        break;
      case 'onAssistantInvoked':
        _lifeCycleObserver.onAssistantInvoked();
        break;
      case 'onAssistantClosed':
        final Map<dynamic, dynamic> _data = call.arguments;
        bool isCancelled = _data["isCancelled"];
        _lifeCycleObserver.onAssistantClosed(isCancelled);
        break;
      case 'onAssistantLocaleChanged':
        final Map<dynamic, dynamic> _data = call.arguments;
        Map<String, String> locale = new HashMap();
        locale["country"] = _data["country"];
        locale["language"] = _data["language"];
        _lifeCycleObserver.onAssistantLocaleChanged(locale);
        break;
      case 'onUnrecognisedUtterance':
        final Map<dynamic, dynamic> _data = call.arguments;
        String utterance = _data["utterance"];
        _lifeCycleObserver.onUnrecognisedUtterance(utterance);
        break;
      case 'onUtteranceDetected':
        final Map<dynamic, dynamic> _data = call.arguments;
        String utterance = _data["utterance"];
        _lifeCycleObserver.onUtteranceDetected(utterance);
        break;
      case 'onOnboardingSuccess':
        _lifeCycleObserver.onOnboardingSuccess();
        break;
      case 'onOnboardingFailure':
        _lifeCycleObserver.onOnboardingFailure();
        break;
      case 'onOnboardingFailure':
        _lifeCycleObserver.onOnboardingFailure();
        break;
      case 'onMicPermissionDenied':
        _lifeCycleObserver.onMicPermissionDenied();
        break;
      default:
        print(
            'SlangRetailAssistant: Ignoring invoke from native. This normally shouldn\'t happen.');
    }
  }

  static bool _validateEnvironment() {
    return !Platform.isIOS;
  }

  static void initialize(AssistantConfiguration assistantConfiguration) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod(
        "initialize", assistantConfiguration._covertToHashmap());
    _channel.setMethodCallHandler(_methodCallHandler);
  }

  static void setAction(RetailAssistantAction action) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("setAction");
    _action = action;
  }

  static void setLifecycleObserver(
      RetailAssistantLifeCycleObserver lifeCycleObserver) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("setLifecycleObserver");
    _lifeCycleObserver = lifeCycleObserver;
  }

  static void startConversation(AssistantUserJourney assistantUserJourney) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod(
        "startConversation", {"userJourney": assistantUserJourney.stringValue});
  }

  static void notifyNonVoiceUserJourney(
      AssistantUserJourney assistantUserJourney,
      Map<String, String> journeyInfo) {
    if (!_validateEnvironment()) {
      return;
    }
    _channel.invokeMapMethod("notifyNonVoiceUserJourney", {
      "userJourney": assistantUserJourney.stringValue,
      "userJourneyInfo": journeyInfo
    });
  }

  static void notifyNonVoiceSearch(String searchTerm) {
    if (!_validateEnvironment()) {
      return;
    }
    notifyNonVoiceUserJourney(
        AssistantUserJourney.SEARCH, {"searchString": searchTerm});
  }

  static void cancelSession() {
    _channel.invokeMapMethod("cancelSession");
  }

  static _AssistantUI getUI() {
    return _AssistantUI(_channel);
  }
}

class RetailAssistantAction {
  SearchAppState onSearch(
      SearchInfo searchInfo, SearchUserJourney searchUserJourney) {
    return SearchAppState.UNSUPPORTED;
  }

  NavigationAppState onNavigation(NavigationInfo navigationInfo,
      NavigationUserJourney navigationUserJourney) {
    return NavigationAppState.UNSUPPORTED;
  }

  OrderManagementAppState onOrderManagement(OrderInfo orderInfo,
      OrderManagementUserJourney orderManagementUserJourney) {
    return OrderManagementAppState.UNSUPPORTED;
  }

  void onAssistantError(Map<String, String> assistantError) {}
}

class RetailAssistantLifeCycleObserver {
  void onAssistantInitSuccess() {}

  void onAssistantInitFailure(String description) {}

  void onAssistantInvoked() {}

  void onAssistantClosed(bool isCancelled) {}

  void onAssistantLocaleChanged(Map<String, String> locale) {}

  void onUnrecognisedUtterance(String utterance) {}

  void onUtteranceDetected(String utterance) {}

  void onOnboardingSuccess() {}

  void onOnboardingFailure() {}

  void onMicPermissionDenied() {}
}

enum AssistantUserJourney { SEARCH, NAVIGATION, ORDER_MANAGEMENT }

extension AssistantUserJourneyExtention on AssistantUserJourney {
  String get stringValue {
    switch (this) {
      case AssistantUserJourney.SEARCH:
        return 'retail_search';
      case AssistantUserJourney.NAVIGATION:
        return 'retail_navigation';
      case AssistantUserJourney.ORDER_MANAGEMENT:
        return 'retail_order_management';
      default:
        return "retail_search";
    }
  }
}

enum AssistantSubDomain { GROCERY, PHARMACY, CUSTOM }

extension AssistantSubDomainExtension on AssistantSubDomain {
  String get stringValue {
    switch (this) {
      case AssistantSubDomain.GROCERY:
        return 'grocery';
      case AssistantSubDomain.PHARMACY:
        return 'pharmacy';
      case AssistantSubDomain.CUSTOM:
        return 'custom';
      default:
        return "grocery";
    }
  }
}

enum AssistantEnvironment { STAGING, PRODUCTION }

extension AssistantEnvironmentExtension on AssistantEnvironment {
  String get stringValue {
    switch (this) {
      case AssistantEnvironment.STAGING:
        return 'STAGING';
      case AssistantEnvironment.PRODUCTION:
        return 'PRODUCTION';
      default:
        return "STAGING";
    }
  }
}
